==> evlan-0.3.3.tbz2: evlan-0.3.3/evlan-vm-0.3.3/src/framework/FrontEnd.cpp <==
/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2003/04/11 (KV)
 *  + Imported code from Evlan.
 */

// ===================================================================

#include "FrontEnd.h"
#include <map>
#include <vector>
#include <iostream>
#include <sstream>
#include <framework/util/Vector.h>

#ifdef PLATFORM_WIN32
#define _WIN32_WINNT 0x0500
#include <windows.h>
#include <winsock.h>
#include <framework/internal/win32/SystemImpl.h>
#undef ERROR

#elif defined(PLATFORM_FREEBSD)
#include <framework/internal/freebsd/SystemImpl.h>
#endif

#ifdef PLATFORM_POSIX
#include <unistd.h>
#endif

using namespace std;

namespace framework {

// class members =====================================================
FrontEnd::UsageError::~UsageError()
{
}

FrontEnd::~FrontEnd()
{
}

namespace
{
   ostream& operator<<(ostream& os, String str)
   {
      if(str != null)
         os.write(&str.getBuffer()[0], str.getLength());
      return os;
   }

   template<typename Type>
   int findInMap(map<Type, int>& switchMap, Type key)
   {
      typename map<Type, int>::iterator i;
      i = switchMap.find(key);
      if(i == switchMap.end())
         return -1;
      else
         return i->second;
   }

   enum ParameterType {SWITCH, SHORTHAND_SWITCH, ARGUMENT};
   struct Parameter
   {
      ParameterType mType;
      String mData;

      inline Parameter(ParameterType type, String data)
         : mType(type), mData(data) {}
   };

   String locateExecutable(String name)
   {
#ifdef PLATFORM_WIN32
      FW_MAKE_CSTRING(nameCString, name);
      Array<char> buffer(MAX_PATH);
      char* filePart;
      int size = SearchPath(null, nameCString, ".exe", MAX_PATH, &buffer[0], &filePart);
      if(size > MAX_PATH)
      {
         buffer = Array<char>(size);
         size = SearchPath(null, nameCString, ".exe", size, &buffer[0], &filePart);
      }
      if(size == 0)
         FW_THROW(spnew Win32Error($("SearchPath")));

      buffer = buffer.getInterval(0, size);

      return String(buffer.toConst()).copy();

#else
      int pos = name.findLastOf('/');

      if(pos == -1)
      {
         const char* pathEnvironment = getenv("PATH");
         while(pathEnvironment != null)
         {
            const char* colon = strchr(pathEnvironment, ':');

            String tryPath;
            if(colon == null)
            {
               tryPath = String(pathEnvironment);
               pathEnvironment = null;
            }
            else
            {
               tryPath = String(pathEnvironment, colon - pathEnvironment);
               pathEnvironment = colon + 1;
            }

            if(tryPath.getLength() == 0)
               continue;

            if(tryPath[tryPath.getLength() - 1] != '/')
               tryPath += '/';
            tryPath += name;

            FW_MAKE_CSTRING(tryPathCString, tryPath);

            if(access(tryPathCString, F_OK) == 0)
               return tryPath;
         }
         FW_THROW(spnew FatalError($("Could not locate the executable.  Which is odd because it's running and all, but, yeah...")));
      }
      else
      {
         if(name[0] != '/')
         {
            char* cwd = getcwd(null, 0);
            String currentDirectory(cwd);
            free(cwd);

            if(currentDirectory.getLength() == 0
            || currentDirectory[currentDirectory.getLength() - 1] != '/')
               currentDirectory += '/';

            name = currentDirectory + name;
         }

         FW_MAKE_CSTRING(nameCString, name);
         if(access(nameCString, F_OK) == 0)
            return name;
         FW_THROW(spnew FatalError($("Could not locate the executable.  Which is odd because it's running and all, but, yeah...")));
      }
#endif
   }
}

int internalMain(FrontEndPtr (*makeFrontEnd)(), int numParams, char* argv[])
{
   int status = 0;

#ifdef TRACK_MEMORY
   MemoryManager::trackAllocations();
#endif

   FrontEnd::ProgramInfo programInfo;

   stringstream errorMessage;

   try
   {
#ifdef PLATFORM_WIN32
      WSADATA junk;
      if(WSAStartup(MAKEWORD(2,2), &junk) != 0)
         FW_THROW(spnew Win32Error($("WSAStartup")));
#endif

      FrontEndPtr frontEnd;
      FrontEnd::StartupInfo startupInfo;
      String executablePath;
      bool showHelp = false;

      {
         executablePath = locateExecutable(String::makeStatic(argv[0]));
         Exception::setExecutablePath(executablePath);

         frontEnd = makeFrontEnd();
         programInfo = frontEnd->getProgramInfo();

         //The following hunk of crappy code takes care of parsing the arguments,
         // telling the front end about them, and then, calling frontEnd->run().
         ++argv;
         --numParams;

         vector<Parameter> params;
         params.reserve(numParams);

         for(int i = 0; i < numParams; i++)
         {
            if(argv[i][0] == '-')
            {
               if(argv[i][1] == '\0')
                  params.push_back(Parameter(ARGUMENT, $("-")));
               else if(argv[i][1] == '-')
               {
                  //long switch
                  if(argv[i][2] == '\0')
                     params.push_back(Parameter(SWITCH, $("--")));
                  else
                  {
                     String name = String::makeStatic(argv[i] + 2);
                     if(name == "help")
                        showHelp = true;
                     else
                        params.push_back(Parameter(SWITCH, name));
                  }
               }
               else
               {
                  //short switch
                  int pos = 1;
                  while(argv[i][pos] != '\0')
                  {
                     if(argv[i][pos] == 'h')
                        showHelp = true;
                     else
                     {
                        params.push_back(
                           Parameter(SHORTHAND_SWITCH,
                                     String::makeStatic(argv[i] + pos, 1)));
                     }
                     ++pos;
                  }
               }
            }
            else
               params.push_back(Parameter(ARGUMENT, String::makeStatic(argv[i])));
         }

         map<String, int> switchMap;
         map<char, int>   shorthandMap;
         util::Vector<FrontEnd::Switch> switches;

         for(int i = 0; i < programInfo.switchSyntax.getSize(); i++)
         {
            if(programInfo.switchSyntax[i].name != null)
               switchMap[programInfo.switchSyntax[i].name] = i;
            if(programInfo.switchSyntax[i].shorthand != '\0')
               shorthandMap[programInfo.switchSyntax[i].shorthand] = i;
         }

         Array<String> others(numParams);
         int leftoverPos = 0;
         int argumentPos = numParams;
         FrontEnd::Switch theSwitch;

         int pos = 0;
         while(pos < (int)params.size())
         {
            if(params[pos].mType == ARGUMENT)
               others[leftoverPos++] = params[pos].mData;
            else
            {
               if(params[pos].mType == SWITCH)
               {
                  theSwitch.identifier = findInMap(switchMap, params[pos].mData);
                  if(theSwitch.identifier == -1)
                  {
                     FW_THROW(spnew FrontEnd::UsageError(
                        $("Switch --") + params[pos].mData + $(" not recognized")));
                  }
               }
               else
               {
                  theSwitch.identifier = findInMap(shorthandMap, params[pos].mData[0]);
                  if(theSwitch.identifier == -1)
                  {
                     FW_THROW(spnew FrontEnd::UsageError(
                        $("Switch -") + params[pos].mData + $(" not recognized")));
                  }
               }

               if(programInfo.switchSyntax[theSwitch.identifier].numArguments > 0)
               {
                  if(pos + programInfo.switchSyntax[theSwitch.identifier].numArguments < (int)params.size())
                  {
                     argumentPos -= programInfo.switchSyntax[theSwitch.identifier].numArguments;
                     theSwitch.arguments = others.getInterval(argumentPos,
                        argumentPos + programInfo.switchSyntax[theSwitch.identifier].numArguments).toConst();

                     Array<String> arguments(programInfo.switchSyntax[theSwitch.identifier].numArguments);
                     for(int i = 0; i < arguments.getSize(); i++)
                     {
                        if(params[pos + i + 1].mType != ARGUMENT)
                        {
                           arguments = null;
                           break;
                        }
                        arguments[i] = params[pos + i + 1].mData;
                     }
                     theSwitch.arguments = arguments.toConst();
                  }

                  if(theSwitch.arguments == null)
                  {
                     if(params[pos].mType == SWITCH)
                     {
                        FW_THROW(spnew FrontEnd::UsageError(
                           $("Switch --") + params[pos].mData + $(" takes ") +
                           String(programInfo.switchSyntax[theSwitch.identifier].numArguments) +
                           $(" argument(s).")));
                     }
                     else
                     {
                        FW_THROW(spnew FrontEnd::UsageError(
                           $("Switch -") + params[pos].mData + $(" takes ") +
                           String(programInfo.switchSyntax[theSwitch.identifier].numArguments) +
                           $(" argument(s).")));
                     }
                  }

                  pos += programInfo.switchSyntax[theSwitch.identifier].numArguments;
               }

               switches.pushBack(theSwitch);
               theSwitch.arguments = null;
            }

            ++pos;
         }

         startupInfo.switches = switches.getArray().toConst();
         startupInfo.arguments = others.getInterval(0, leftoverPos).toConst();
      }

      if(showHelp)
         cout << "Usage:\n" << programInfo.helpMessage << endl;
      else
      {
#ifdef PLATFORM_WIN32
         internal::win32::startSystem(frontEnd, startupInfo, programInfo.name,
                                      executablePath);
#elif defined(PLATFORM_FREEBSD)
         internal::freebsd::startSystem(frontEnd, startupInfo, programInfo.name,
                                        executablePath);
#else
#error "port this"
#endif
      }
   }
   FW_CATCH_START
      FW_CATCH(FrontEnd::UsageError, e)
      {
         errorMessage << "USAGE ERROR: " << e->getDescription() << endl;
         errorMessage << "Usage:\n" << programInfo.helpMessage << endl;
         status = -1;
      }
      FW_CATCH(Exception, e)
      {
         cout.flush();
         errorMessage << "Uncaught exception:  ";
         while(true)
         {
            errorMessage << e->getDescription();
#ifdef DEBUG
            errorMessage << "\nStack trace:";
            errorMessage << endl;
            errorMessage << e->getStackTrace();
#else
#endif
            e = e->getCause();
            if(e == null)
               break;
            errorMessage << "\nCaused by: ";
         }
         status = -1;
      }
   FW_CATCH_END
   catch(const bad_alloc& e)
   {
      cout.flush();
      errorMessage << "Out of memory.  Stack trace:" << endl;
      int stackSize;
      void** stack = MemoryManager::getBadAllocStackTrace(&stackSize);
      errorMessage << Exception::getStackTrace(stack, stackSize);
      status = -1;
   }
   catch(const exception& e)
   {
      cout.flush();
      errorMessage << "Uncaught C++ library exception: " << e.what() << endl;
      status = -1;
   }
   catch(...)
   {
      cout.flush();
      errorMessage << "Uncaught exception of unknown type." << endl;
      status = -1;
   }

#ifdef PLATFORM_WIN32
   WSACleanup();
#endif

#ifdef TRACK_MEMORY
   MemoryManager::stopTracking();
   if(MemoryManager::isDataAllocated())
   {
      cerr << "Possible memory leaks detected.  Report written to \"memory_leaks.log\"." << endl;
      MemoryManager::dumpDebugInfo("memory_leaks.log");
   }
#endif

   if(status != 0)
   {
#ifdef PLATFORM_WIN32
      MessageBox(GetConsoleWindow(), errorMessage.str().c_str(), "Evlan: Fatal Error", MB_OK | MB_ICONERROR | MB_SYSTEMMODAL);
#else
      cerr << errorMessage.str();
      cerr.flush();
#endif
   }

   return status;
}

} //namespace framework

==> evlan-0.3.4.tbz2: evlan-0.3.4/evlan-vm-0.3.4/src/framework/FrontEnd.cpp <==
/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2003/04/11 (KV)
 *  + Imported code from Evlan.
 */

// ===================================================================

#include "FrontEnd.h"
#include <map>
#include <vector>
#include <iostream>
#include <sstream>
#include <framework/util/Vector.h>

#ifdef PLATFORM_WIN32
#define _WIN32_WINNT 0x0500
#include <windows.h>
#include <winsock.h>
#include <framework/internal/win32/SystemImpl.h>
#undef ERROR

#elif defined(PLATFORM_FREEBSD)
#include <framework/internal/freebsd/SystemImpl.h>
#elif defined(PLATFORM_DARWIN)
#include <framework/internal/darwin/SystemImpl.h>
#elif defined(PLATFORM_POSIX)
#include <framework/internal/posix/SystemImpl.h>
#endif

#ifdef PLATFORM_POSIX
#include <unistd.h>
#endif

using namespace std;

namespace framework {

// class members =====================================================
FrontEnd::UsageError::~UsageError()
{
}

FrontEnd::~FrontEnd()
{
}

namespace
{
   ostream& operator<<(ostream& os, String str)
   {
      if(str != null)
         os.write(&str.getBuffer()[0], str.getLength());
      return os;
   }

   template<typename Type>
   int findInMap(map<Type, int>& switchMap, Type key)
   {
      typename map<Type, int>::iterator i;
      i = switchMap.find(key);
      if(i == switchMap.end())
         return -1;
      else
         return i->second;
   }

   enum ParameterType {SWITCH, SHORTHAND_SWITCH, ARGUMENT};
   struct Parameter
   {
      ParameterType mType;
      String mData;

      inline Parameter(ParameterType type, String data)
         : mType(type), mData(data) {}
   };

   String locateExecutable(String name)
   {
#ifdef PLATFORM_WIN32
      FW_MAKE_CSTRING(nameCString, name);
      Array<char> buffer(MAX_PATH);
      char* filePart;
      int size = SearchPath(null, nameCString, ".exe", MAX_PATH, &buffer[0], &filePart);
      if(size > MAX_PATH)
      {
         buffer = Array<char>(size);
         size = SearchPath(null, nameCString, ".exe", size, &buffer[0], &filePart);
      }
      if(size == 0)
         FW_THROW(spnew Win32Error($("SearchPath")));

      buffer = buffer.getInterval(0, size);

      return String(buffer.toConst()).copy();

#else
      int pos = name.findLastOf('/');

      if(pos == -1)
      {
         const char* pathEnvironment = getenv("PATH");
         while(pathEnvironment != null)
         {
            const char* colon = strchr(pathEnvironment, ':');

            String tryPath;
            if(colon == null)
            {
               tryPath = String(pathEnvironment);
               pathEnvironment = null;
            }
            else
            {
               tryPath = String(pathEnvironment, colon - pathEnvironment);
               pathEnvironment = colon + 1;
            }

            if(tryPath.getLength() == 0)
               continue;

            if(tryPath[tryPath.getLength() - 1] != '/')
               tryPath += '/';
            tryPath += name;

            FW_MAKE_CSTRING(tryPathCString, tryPath);

            if(access(tryPathCString, F_OK) == 0)
               return tryPath;
         }
         FW_THROW(spnew FatalError($("Could not locate the executable.  Which is odd because it's running and all, but, yeah...")));
      }
      else
      {
         if(name[0] != '/')
         {
            char* cwd = getcwd(null, 0);
            String currentDirectory(cwd);
            free(cwd);

            if(currentDirectory.getLength() == 0
            || currentDirectory[currentDirectory.getLength() - 1] != '/')
               currentDirectory += '/';

            name = currentDirectory + name;
         }

         FW_MAKE_CSTRING(nameCString, name);
         if(access(nameCString, F_OK) == 0)
            return name;
         FW_THROW(spnew FatalError($("Could not locate the executable.  Which is odd because it's running and all, but, yeah...")));
      }
#endif
   }
}

int internalMain(FrontEndPtr (*makeFrontEnd)(), int numParams, char* argv[])
{
   int status = 0;

#ifdef TRACK_MEMORY
   MemoryManager::trackAllocations();
#endif

   FrontEnd::ProgramInfo programInfo;

   stringstream errorMessage;

   try
   {
#ifdef PLATFORM_WIN32
      WSADATA junk;
      if(WSAStartup(MAKEWORD(2,2), &junk) != 0)
         FW_THROW(spnew Win32Error($("WSAStartup")));
#endif

      FrontEndPtr frontEnd;
      FrontEnd::StartupInfo startupInfo;
      String executablePath;
      bool showHelp = false;

      {
         executablePath = locateExecutable(String::makeStatic(argv[0]));
         Exception::setExecutablePath(executablePath);

         frontEnd = makeFrontEnd();
         programInfo = frontEnd->getProgramInfo();

         //The following hunk of crappy code takes care of parsing the arguments,
         // telling the front end about them, and then, calling frontEnd->run().
         ++argv;
         --numParams;

         vector<Parameter> params;
         params.reserve(numParams);

         for(int i = 0; i < numParams; i++)
         {
            if(argv[i][0] == '-')
            {
               if(argv[i][1] == '\0')
                  params.push_back(Parameter(ARGUMENT, $("-")));
               else if(argv[i][1] == '-')
               {
                  //long switch
                  if(argv[i][2] == '\0')
                     params.push_back(Parameter(SWITCH, $("--")));
                  else
                  {
                     String name = String::makeStatic(argv[i] + 2);
                     if(name == "help")
                        showHelp = true;
                     else
                        params.push_back(Parameter(SWITCH, name));
                  }
               }
               else
               {
                  //short switch
                  int pos = 1;
                  while(argv[i][pos] != '\0')
                  {
                     if(argv[i][pos] == 'h')
                        showHelp = true;
                     else
                     {
                        params.push_back(
                           Parameter(SHORTHAND_SWITCH,
                                     String::makeStatic(argv[i] + pos, 1)));
                     }
                     ++pos;
                  }
               }
            }
            else
               params.push_back(Parameter(ARGUMENT, String::makeStatic(argv[i])));
         }

         map<String, int> switchMap;
         map<char, int>   shorthandMap;
         util::Vector<FrontEnd::Switch> switches;

         for(int i = 0; i < programInfo.switchSyntax.getSize(); i++)
         {
            if(programInfo.switchSyntax[i].name != null)
               switchMap[programInfo.switchSyntax[i].name] = i;
            if(programInfo.switchSyntax[i].shorthand != '\0')
               shorthandMap[programInfo.switchSyntax[i].shorthand] = i;
         }

         Array<String> others(numParams);
         int leftoverPos = 0;
         int argumentPos = numParams;
         FrontEnd::Switch theSwitch;

         int pos = 0;
         while(pos < (int)params.size())
         {
            if(params[pos].mType == ARGUMENT)
               others[leftoverPos++] = params[pos].mData;
            else
            {
               if(params[pos].mType == SWITCH)
               {
                  theSwitch.identifier = findInMap(switchMap, params[pos].mData);
                  if(theSwitch.identifier == -1)
                  {
                     FW_THROW(spnew FrontEnd::UsageError(
                        $("Switch --") + params[pos].mData + $(" not recognized")));
                  }
               }
               else
               {
                  theSwitch.identifier = findInMap(shorthandMap, params[pos].mData[0]);
                  if(theSwitch.identifier == -1)
                  {
                     FW_THROW(spnew FrontEnd::UsageError(
                        $("Switch -") + params[pos].mData + $(" not recognized")));
                  }
               }

               if(programInfo.switchSyntax[theSwitch.identifier].numArguments > 0)
               {
                  if(pos + programInfo.switchSyntax[theSwitch.identifier].numArguments < (int)params.size())
                  {
                     argumentPos -= programInfo.switchSyntax[theSwitch.identifier].numArguments;
                     theSwitch.arguments = others.getInterval(argumentPos,
                        argumentPos + programInfo.switchSyntax[theSwitch.identifier].numArguments).toConst();

                     Array<String> arguments(programInfo.switchSyntax[theSwitch.identifier].numArguments);
                     for(int i = 0; i < arguments.getSize(); i++)
                     {
                        if(params[pos + i + 1].mType != ARGUMENT)
                        {
                           arguments = null;
                           break;
                        }
                        arguments[i] = params[pos + i + 1].mData;
                     }
                     theSwitch.arguments = arguments.toConst();
                  }

                  if(theSwitch.arguments == null)
                  {
                     if(params[pos].mType == SWITCH)
                     {
                        FW_THROW(spnew FrontEnd::UsageError(
                           $("Switch --") + params[pos].mData + $(" takes ") +
                           String(programInfo.switchSyntax[theSwitch.identifier].numArguments) +
                           $(" argument(s).")));
                     }
                     else
                     {
                        FW_THROW(spnew FrontEnd::UsageError(
                           $("Switch -") + params[pos].mData + $(" takes ") +
                           String(programInfo.switchSyntax[theSwitch.identifier].numArguments) +
                           $(" argument(s).")));
                     }
                  }

                  pos += programInfo.switchSyntax[theSwitch.identifier].numArguments;
               }

               switches.pushBack(theSwitch);
               theSwitch.arguments = null;
            }

            ++pos;
         }

         startupInfo.switches = switches.getArray().toConst();
         startupInfo.arguments = others.getInterval(0, leftoverPos).toConst();
      }

      if(showHelp)
         cout << "Usage:\n" << programInfo.helpMessage << endl;
      else
      {
#ifdef PLATFORM_WIN32
         internal::win32::startSystem(frontEnd, startupInfo, programInfo.name,
                                      executablePath);
#elif defined(PLATFORM_FREEBSD)
         internal::freebsd::startSystem(frontEnd, startupInfo, programInfo.name,
                                        executablePath);
#elif defined(PLATFORM_DARWIN)
         internal::darwin::startSystem(frontEnd, startupInfo, programInfo.name,
                                       executablePath);
#elif defined(PLATFORM_POSIX)
         internal::posix::startSystem(frontEnd, startupInfo, programInfo.name,
                                      executablePath);
#else
#error "port this"
#endif
      }
   }
   FW_CATCH_START
      FW_CATCH(FrontEnd::UsageError, e)
      {
         errorMessage << "USAGE ERROR: " << e->getDescription() << endl;
         errorMessage << "Usage:\n" << programInfo.helpMessage << endl;
         status = -1;
      }
      FW_CATCH(Exception, e)
      {
         cout.flush();
         errorMessage << "Uncaught exception:  ";
         while(true)
         {
            errorMessage << e->getDescription();
#ifdef DEBUG
            errorMessage << "\nStack trace:";
            errorMessage << endl;
            errorMessage << e->getStackTrace();
#else
#endif
            e = e->getCause();
            if(e == null)
               break;
            errorMessage << "\nCaused by: ";
         }
         status = -1;
      }
   FW_CATCH_END
   catch(const bad_alloc& e)
   {
      cout.flush();
      errorMessage << "Out of memory.  Stack trace:" << endl;
      int stackSize;
      void** stack = MemoryManager::getBadAllocStackTrace(&stackSize);
      errorMessage << Exception::getStackTrace(stack, stackSize);
      status = -1;
   }
   catch(const exception& e)
   {
      cout.flush();
      errorMessage << "Uncaught C++ library exception: " << e.what() << endl;
      status = -1;
   }
   catch(...)
   {
      cout.flush();
      errorMessage << "Uncaught exception of unknown type." << endl;
      status = -1;
   }

#ifdef PLATFORM_WIN32
   WSACleanup();
#endif

#ifdef TRACK_MEMORY
   MemoryManager::stopTracking();
   if(MemoryManager::isDataAllocated())
   {
      cerr << "Possible memory leaks detected.  Report written to \"memory_leaks.log\"." << endl;
      MemoryManager::dumpDebugInfo("memory_leaks.log");
   }
#endif

   if(status != 0)
   {
#ifdef PLATFORM_WIN32
      MessageBox(GetConsoleWindow(), errorMessage.str().c_str(), "Evlan: Fatal Error", MB_OK | MB_ICONERROR | MB_SYSTEMMODAL);
#else
      cerr << errorMessage.str() << endl;
#endif
   }

   return status;
}

} //namespace framework
